@@ -1,7 +1,7 @@
 .onAttach <-
 function(libname, pkgname) {
     packageStartupMessage(
-        "\nFor more information on the usage of the BEQI2 tool, type:\n", 
+        "For more information on the usage of the BEQI2 tool, type: ", 
         'vignette("BEQI2")'
     )
 }  
